package com.graywolf336.jail;

import com.graywolf336.jail.beans.Jail;
import com.graywolf336.jail.beans.Prisoner;
import com.graywolf336.jail.enums.Lang;
import com.graywolf336.jail.enums.Settings;
import com.graywolf336.jail.events.PrisonerTimeChangeEvent;
import org.bukkit.entity.Player;

import javax.swing.*;

/**
 * Contains all the logic for counting down the time of the prisoners time.
 *
 * @author graywolf336
 * @version 3.0.0
 * @since 2.x.x
 */
public class JailTimer {

    private final JailMain jailMain;
    private Timer timer;
    private Long lastTime;
    private Long afkTime = 0L;

    protected JailTimer(JailMain plugin) {
        this.jailMain = plugin;
        try {
            afkTime = Util.getTime(jailMain.getConfig().getString(Settings.MAXAFKTIME.getPath()));
        } catch (Exception e) {
            jailMain.getLogger().severe("Error while processing the max afk time: " + e.getMessage());
        }

        this.lastTime = System.currentTimeMillis();
        if (jailMain.getConfig().getBoolean(Settings.USEBUKKITTIMER.getPath())) {
            jailMain.getLogger().info("Using the Bukkit Scheduler.");
            jailMain.getServer().getScheduler().runTaskTimerAsynchronously(jailMain, new TimeEvent(), 200, 200);
        } else {
            jailMain.getLogger().info("Using the Java Timer.");
            timer = new Timer(10000, event -> jailMain.getServer().getScheduler().scheduleSyncDelayedTask(jailMain, new TimeEvent()));

            timer.start();
        }

        //Save all the jail information every minute, not every 10 seconds
        jailMain.getServer().getScheduler().runTaskTimerAsynchronously(jailMain, () -> {
            for (Jail j : jailMain.getJailManager().getJails()) {
                jailMain.getJailIO().saveJail(j);
            }
        }, 1200L, 1200L);
    }

    /**
     * Returns the instance of this timer.
     *
     * @return the {@link Timer} instance
     */
    public Timer getTimer() {
        return this.timer;
    }

    private class TimeEvent implements Runnable {
        public void run() {
            long timePassed = System.currentTimeMillis() - lastTime;
            lastTime = System.currentTimeMillis();

            for (Jail jail : jailMain.getJailManager().getJails()) {
                for (Prisoner prisoner : jail.getAllPrisoners().values()) {
                    //only execute this code if the prisoner's time is more than 0 milliseconds
                    //and they don't have any offline pending things
                    if (prisoner.getRemainingTime() > 0 && !prisoner.isOfflinePending()) {
                        final Player player = jailMain.getServer().getPlayer(prisoner.getUUID());

                        //Check if the player is offline
                        if (player == null) {
                            //if they are offline AND the config has counting down the time
                            //while the prisoner is offline, then let's do it
                            if (jailMain.getConfig().getBoolean(Settings.COUNTDOWNTIMEOFFLINE.getPath())) {
                                //Set their remaining time but if it is less than zero, set it to zero
                                long before = prisoner.getRemainingTime();
                                long after = Math.max(0, prisoner.getRemainingTime() - timePassed);

                                PrisonerTimeChangeEvent event = new PrisonerTimeChangeEvent(jail, jail.getCellPrisonerIsIn(prisoner.getUUID()), prisoner, player, before, after);
                                jailMain.getServer().getPluginManager().callEvent(event);

                                if (!event.isCancelled()) {
                                    after = event.getTimeAfterChange();
                                    prisoner.setRemainingTime(after);
                                    if (prisoner.getRemainingTime() == 0) {
                                        jailMain.getPrisonerManager().schedulePrisonerRelease(prisoner);
                                    }
                                }
                            }
                        } else {
                            if (afkTime > 0) {
                                prisoner.setAFKTime(prisoner.getAFKTime() + timePassed);
                                if (prisoner.getAFKTime() > afkTime) {
                                    prisoner.setAFKTime(0);
                                    //This is so we kick players on the main thread
                                    //instead of on the async thread(s), as spigot
                                    //has a protection against this enabled.
                                    jailMain.getServer().getScheduler().scheduleSyncDelayedTask(jailMain, () -> player.kickPlayer(Lang.AFKKICKMESSAGE.get()));
                                }
                            }

                            //The prisoner isn't offline, so let's count down
                            //Set their remaining time but if it is less than zero, set it to zero
                            long before = prisoner.getRemainingTime();
                            long after = Math.max(0, prisoner.getRemainingTime() - timePassed);

                            PrisonerTimeChangeEvent event = new PrisonerTimeChangeEvent(jail, jail.getCellPrisonerIsIn(prisoner.getUUID()), prisoner, player, before, after);
                            jailMain.getServer().getPluginManager().callEvent(event);

                            if (!event.isCancelled()) {
                                after = event.getTimeAfterChange();
                                prisoner.setRemainingTime(after);
                                if (prisoner.getRemainingTime() == 0) {
                                    jailMain.getPrisonerManager().schedulePrisonerRelease(prisoner);
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
